/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.gagravarr.flac;

import java.io.IOException;
import java.io.OutputStream;


/**
 * Any Flac Metadata Block we don't explicitly handle
 */
public class FlacUnhandledMetadataBlock extends FlacMetadataBlock {
   private byte[] data;
   
   public FlacUnhandledMetadataBlock(byte type, byte[] data) {
      super(type);
      this.data = data;
   }
   
	protected void write(OutputStream out) throws IOException {
	   out.write(data);
	}
}
