/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.gagravarr.flac.tools;

import java.io.File;

import org.gagravarr.flac.FlacFile;
import org.gagravarr.vorbis.tools.VorbisLikeCommentTool;
import org.gagravarr.vorbis.tools.VorbisLikeCommentTool.Command.Commands;

/**
 * A class for listing (but not yet editing) Comments (Tags) within a
 *  FLAC File, much like the vorbiscomments program (but FLAC)
 */
public class FlacCommentTool extends VorbisLikeCommentTool {
    public static void main(String[] args) throws Exception {
        Command command = processArgs(args, "FlacComment");
        
        FlacFile op = FlacFile.open(new File(command.inFile));
        
        if (command.command == Commands.List) {
            listTags(op.getTags());
        } else {
            throw new IllegalArgumentException("Writing is not (yet) supported for FLAC");
        }
    }
}
